@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.9 2001/01/10 07:42:53 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.10 2001/01/15 13:43:46 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -328,7 +328,7 @@ void ip_vs_conn_put(struct ip_vs_conn *cp)
 /*
  *	Timeout table[state]
  */
-static struct ip_vs_timeout_table vs_timeout_table = {
+struct ip_vs_timeout_table vs_timeout_table = {
 	ATOMIC_INIT(0),	/* refcnt */
 	0,		/* scale  */
 	{
