@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.9 2001/01/10 07:42:53 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.10 2001/01/15 13:43:46 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -62,9 +62,10 @@ static spinlock_t __ip_vs_dropentry_lock = SPIN_LOCK_UNLOCKED;
 /* lock for drop packet handling */
 static spinlock_t __ip_vs_droppacket_lock = SPIN_LOCK_UNLOCKED;
 
-static int ip_vs_drop_rate = 0;
-static int ip_vs_drop_counter = 0;
-static atomic_t ip_vs_dropentry = ATOMIC_INIT(0);
+/* 1/rate drop and drop-entry variables */
+int ip_vs_drop_rate = 0;
+int ip_vs_drop_counter = 0;
+atomic_t ip_vs_dropentry = ATOMIC_INIT(0);
 
 /* sysctl variables */
 static int sysctl_ip_vs_drop_entry = 0;
@@ -72,8 +73,8 @@ static int sysctl_ip_vs_drop_packet = 0;
 static int sysctl_ip_vs_secure_tcp = 0;
 static int sysctl_ip_vs_amemthresh = 1024;
 static int sysctl_ip_vs_am_droprate = 10;
-static int sysctl_use_dst = 1;
-static int sysctl_ip_vs_cache_bypass = 0;
+int sysctl_use_dst = 1;
+int sysctl_ip_vs_cache_bypass = 0;
 
 #ifdef CONFIG_IP_VS_DEBUG
 static int sysctl_ip_vs_debug_level = 0;
