@@ -1,9 +1,10 @@
 /*
  *      ipvsadm - IP Virtual Server ADMinistration program
+ *                for IPVS NetFilter Module in kernel 2.4
  *
- *      Version: $Id: ipvsadm.c,v 1.4 2000/11/26 04:35:39 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.5 2001/01/11 14:14:59 wensong Exp $
  *
- *      Authors: Wensong Zhang <wensong@iinchina.net>
+ *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *               Peter Kese <peter.kese@ijs.si>
  *
  *      This program is based on ippfvsadm.
@@ -44,6 +45,11 @@
  *                                with popt problem fixed.
  *        Wensong Zhang       :   split the process_options and make
  *                                two versions of parse_options.
+ *        Horms               :   attempting to save or restore when
+ *        		      :   compiled against getopt_long now results
+ *        		          in an informative error message rather
+ *        		          than the usage information
+ *        Horms               :   added -v option
  *
  *
  *      ippfvsadm - Port Fowarding & Virtual Server ADMinistration program
@@ -111,8 +117,8 @@
 #define IPVS_OPTION_PROCESSING          "getopt_long"
 #endif
 
-#define IPVSADM_VERSION_NO              "v1.12"
-#define IPVSADM_VERSION_DATE            "2000/09/06"
+#define IPVSADM_VERSION_NO              "v" VERSION
+#define IPVSADM_VERSION_DATE            "2001/01/11"
 #define IPVSADM_VERSION         IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
 
 #define MINIMUM_IPVS_VERSION_MAJOR      0
@@ -159,6 +165,8 @@ int parse_netmask(char *buf, u_int32_t *addr);
 int parse_timeout(char *buf, unsigned *timeout);
 
 void usage_exit(const char *program, const int exit_status);
+void version_exit(int exit_status);
+void version(FILE *stream);
 void fail(int err, char *text);
 void check_ipvs_version(void);
 void list_vs(unsigned int format);
@@ -228,6 +236,8 @@ int parse_options(int argc, char **argv, int reading_stdin,
         {"set", 's', POPT_ARG_NONE, NULL, 's'};
         struct poptOption help_option =
         {"help", 'h', POPT_ARG_NONE, NULL, 'h'};
+        struct poptOption version_option =
+        {"version", 'v', POPT_ARG_NONE, NULL, 'v'};
         struct poptOption read_stdin_option =
         {"restore", 'R', POPT_ARG_NONE, NULL, 'R'};
         struct poptOption write_stdout_option =
@@ -278,6 +288,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
 		delete_server_option,
                 set_option,
 		help_option,
+		version_option,
 		read_stdin_option,
 		write_stdout_option,
 		NULL_option
@@ -339,8 +350,8 @@ int parse_options(int argc, char **argv, int reading_stdin,
 		NULL_option
 	};
 
-	context= poptGetContext("ipvsadm", argc, (const char **)argv,
-                                options_main, 0);
+	context = poptGetContext("ipvsadm", argc, (const char **)argv,
+                                 options_main, 0);
 
         if ((cmd = poptGetNextOpt(context)) < 0)
                 usage_exit(argv[0], -1);
@@ -394,6 +405,9 @@ int parse_options(int argc, char **argv, int reading_stdin,
 	case 'h':
                 usage_exit(argv[0], 0);
 		break;
+	case 'v':
+                version_exit(0);
+		break;
         default:
                 usage_exit(argv[0], -1);
         }
@@ -473,12 +487,12 @@ int parse_options(int argc, char **argv, int reading_stdin,
                 case 'R':
                         if (destination_set)
                                 fail(2, "Destination already set");
-                        destination_set=1;
+                        destination_set = 1;
                         parse = parse_service(optarg,
                                               ur->protocol,
                                               &ur->daddr, 
                                               &ur->dport);
-                        if (!(parse&SERVICE_ADDR)) 
+                        if (!(parse & SERVICE_ADDR)) 
 				fail(2, "illegal real server "
                                      "address[:port] specified");
                         /* copy vport to dport if none specified */
@@ -488,19 +502,19 @@ int parse_options(int argc, char **argv, int reading_stdin,
                 case 'i':
 			if(forward_set)
                         	fail(2, "multiple forward mechanims set");
-			forward_set=1;
+			forward_set = 1;
                         ur->conn_flags = IP_VS_CONN_F_TUNNEL;
                         break;
                 case 'g':
 			if(forward_set)
                         	fail(2, "multiple forward mechanims set");
-			forward_set=1;
+			forward_set = 1;
                         ur->conn_flags = IP_VS_CONN_F_DROUTE;
                         break;
                 case 'm':
 			if(forward_set)
                         	fail(2, "multiple forward mechanims set");
-			forward_set=1;
+			forward_set = 1;
                         ur->conn_flags = 0;
                         break;
                 case 'w':
@@ -564,7 +578,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
 	 * in the argument list
 	 */
 	if (ur->vs_flags == IP_VS_SVC_F_PERSISTENT){
-		optarg = (char*)poptGetArg(context);
+		optarg = (char *)poptGetArg(context);
        		parse = parse_timeout(optarg, &ur->timeout);
        		if (parse == 0)
                         fail(2, "illegal timeout for persistent service");
@@ -614,6 +628,9 @@ int parse_options(int argc, char **argv, int reading_stdin,
         	{"edit-server", 0, 0, 'e'},
         	{"delete-server", 0, 0, 'd'},
         	{"help", 0, 0, 'h'},
+        	{"version", 0, 0, 'v'},
+        	{"save", 0, 0, 'S'},
+        	{"restore", 0, 0, 'R'},
         	{"set", 1, 0, 's'},
         	{"tcp-service", 1, 0, 't'},
         	{"udp-service", 1, 0, 'u'},
@@ -637,10 +654,10 @@ int parse_options(int argc, char **argv, int reading_stdin,
 	extern int opterr;
 	extern int optopt;
 
-	/* Re-process the arguments each time options is called*/
+	/* Re-process the arguments each time options is called */
 	optind = 1;
 
-	if ((cmd = getopt_long(argc, argv, "AEDCaedlLhs:",
+	if ((cmd = getopt_long(argc, argv, "AEDCSRaedlLhvs:",
                                long_options, NULL)) == EOF)
 		usage_exit(argv[0], -1);
 
@@ -698,6 +715,23 @@ int parse_options(int argc, char **argv, int reading_stdin,
 		break;
 	case 'h':
                 usage_exit(argv[0], 0);
+	case 'v':
+                version_exit(0);
+		break;
+        case 'S': 
+		fprintf(stderr, 
+			"ipvsadm: Invalid option: -S or --save\n"
+			"  Saving of ipvsadm rules is only supported when\n"
+			"  ipvsadm is compiled against libpopt.\n");
+		exit(-1);
+		break;
+	case 'R':
+		fprintf(stderr, 
+			"ipvsadm: Invalid option: -R or --restore\n"
+			"  Restoring ipvsadm rules is only supported when \n"
+			"  ipvsadm is compiled against libpopt.\n");
+		exit(-1);
+		break;
         default:
                 usage_exit(argv[0], -1);
         }
@@ -766,7 +800,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
                 case 'R':
                         if (destination_set)
                                 fail(2, "Destination already set");
-                        destination_set=1;
+                        destination_set = 1;
                         parse = parse_service(optarg,
                                               ur->protocol,
                                               &ur->daddr, 
@@ -781,19 +815,19 @@ int parse_options(int argc, char **argv, int reading_stdin,
                 case 'i':
 			if(forward_set)
                         	fail(2, "multiple forward mechanims set");
-			forward_set=1;
+			forward_set = 1;
                         ur->conn_flags = IP_VS_CONN_F_TUNNEL;
                         break;
                 case 'g':
 			if(forward_set)
                         	fail(2, "multiple forward mechanims set");
-			forward_set=1;
+			forward_set = 1;
                         ur->conn_flags = IP_VS_CONN_F_DROUTE;
                         break;
                 case 'm':
 			if(forward_set)
                         	fail(2, "multiple forward mechanims set");
-			forward_set=1;
+			forward_set = 1;
                         ur->conn_flags = 0;
                         break;
                 case 'w':
@@ -979,7 +1013,7 @@ int parse_netmask(char *buf, u_int32_t *addr)
 {
         struct in_addr inaddr;
 
-	if(buf==NULL)
+	if(buf == NULL)
 		return 0;
         
         if (inet_aton(buf, &inaddr) != 0)
@@ -1004,7 +1038,7 @@ int parse_fwmark(char *buf, u_int32_t *fwmark)
 
         if ((tmpl=string_to_number(buf, 0, 0)) == -1)
                 return 0;
-        *fwmark=tmpl;
+        *fwmark = tmpl;
 
         return 1;
 }
@@ -1038,10 +1072,10 @@ int parse_service(char *buf, u_int16_t proto, u_int32_t *addr, u_int16_t *port)
         else                
                 return SERVICE_NONE;
 
-        result|=SERVICE_ADDR;
+        result |= SERVICE_ADDR;
         
         if (portp != NULL){
-                result|=SERVICE_PORT;
+                result |= SERVICE_PORT;
         
                 if ((portn=string_to_number(portp+1, 0, 65535)) != -1)
                         *port = htons(portn);
@@ -1085,22 +1119,21 @@ void usage_exit(const char *program, const int exit_status) {
         else
                 stream = stdout;
 
+	version(stream);
         fprintf(stream,
-                "ipvsadm " IPVSADM_VERSION " (compiled with "
-                IPVS_OPTION_PROCESSING " and IPVS v%d.%d.%d)\n"
-                "Usage: %s -[A|E] -[t|u|f] service-address [-s scheduler] [-p [timeout]] [-M netmask]\n"
-                "       %s -D -[t|u|f] service-address\n"
-                "       %s -C\n"
+                "Usage:\n"
+                "  %s -[A|E] -[t|u|f] service-address [-s scheduler] [-p [timeout]] [-M netmask]\n"
+                "  %s -D -[t|u|f] service-address\n"
+                "  %s -C\n"
 #ifdef HAVE_POPT
-                "       %s -R\n"
-                "       %s -S [-n]\n"
+                "  %s -R\n"
+                "  %s -S [-n]\n"
 #endif
-                "       %s -[a|e] -[t|u|f] service-address -[r|R] server-address [-g|-i|-m] [-w weight]\n"
-                "       %s -d -[t|u|f] service-address -[r|R] server-address\n"
-                "       %s -[L|l] [-n]\n"
-                "       %s -s tcp tcpfin udp\n"
-                "       %s -h\n\n",
-                NVERSION(IP_VS_VERSION_CODE),                     
+                "  %s -[a|e] -[t|u|f] service-address -[r|R] server-address [-g|-i|-m] [-w weight]\n"
+                "  %s -d -[t|u|f] service-address -[r|R] server-address\n"
+                "  %s -[L|l] [-n]\n"
+                "  %s -s tcp tcpfin udp\n"
+                "  %s -h\n\n",
 #ifdef HAVE_POPT
                 program, program,
 #endif
@@ -1139,7 +1172,7 @@ void usage_exit(const char *program, const int exit_status) {
                 "  --gatewaying   -g                   gatewaying (direct routing) (default)\n"
                 "  --ipip         -i                   ipip encapsulation (tunneling)\n"
                 "  --masquerading -m                   masquerading (NAT)\n"
-                "  --weight:      -w <weight>          capacity of real server\n"
+                "  --weight       -w <weight>          capacity of real server\n"
                 "  --numeric      -n                   numeric output of addresses and ports\n",
                 DEF_SCHED);
         
@@ -1147,6 +1180,28 @@ void usage_exit(const char *program, const int exit_status) {
 }
 
 
+void version_exit(const int exit_status) {
+        FILE *stream;
+
+        if (exit_status != 0)
+                stream = stderr;
+        else
+                stream = stdout;
+
+        version(stream);
+        
+        exit(exit_status);
+}
+
+
+void version(FILE *stream){
+        fprintf(stream,
+                "ipvsadm " IPVSADM_VERSION " (compiled with "
+                IPVS_OPTION_PROCESSING " and IPVS v%d.%d.%d)\n",
+		NVERSION(IP_VS_VERSION_CODE));
+}
+
+
 void fail(int err, char *text) {
         printf("%s\n",text);
         exit(err);
@@ -1189,7 +1244,7 @@ void check_ipvs_version(void)
                                 MINIMUM_IPVS_VERSION_MINOR,
                                 MINIMUM_IPVS_VERSION_PATCH);
 		}
-		buffer[strlen(buffer)-1]='\0';
+		buffer[strlen(buffer)-1] = '\0';
 	}
         fclose(handle);
 }
@@ -1398,7 +1453,7 @@ void print_vsinfo(char *buf, unsigned int format)
                 free(dname);
         } else if (buf[0] == 'F') {
                 /* fwmark virtual service entry */
-                if ((n = sscanf(buf, "%s %X %s %s %d %lX", protocol, 
+                if ((n = sscanf(buf, "%s %X %s %s %u %lX", protocol, 
                                 &fwmark, scheduler, flags, 
                                 &timeout, &temp2)) == -1)
                         exit(1);
@@ -1431,7 +1486,7 @@ void print_vsinfo(char *buf, unsigned int format)
                 /* TCP/UDP virtual service entry  */
                 fwmark=0;  /* Reset firewall mark to unused */
                 
-                if ((n = sscanf(buf, "%s %lX:%hX %s %s %d %lX",
+                if ((n = sscanf(buf, "%s %lX:%hX %s %s %u %lX",
                                 protocol, &temp, &vport, scheduler,
                                 flags, &timeout, &temp2)) == -1)
                         exit(1);
